<?php

/*
 * This file is part of the Frenzy WordPress plugin.
 *
 * (c) Pressware <tom@pressware.co>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Frenzy\API;

use WP_Error;
use WP_Http;

/**
 * The Frenzy API Client.
 *
 * If status = already in DB you call the old endpoint
 * If status = no products, stop calling the API altogether
 * If status = error, it means the article is still processing most likely
 *
 * @author Pressware, LLC <support@pressware.co>
 */
class Client {
	/**
	 * Successful http request status code lower limit
	 *
	 * @var int
	 */
	const SUCCESSFUL_LOWER = 200;

	/**
	 * Successful http request status code upper limit
	 *
	 * @var int
	 */
	const SUCCESSFUL_UPPER = 300;

	/**
	 * Base URL for all Comment IQ API endpoints.
	 *
	 * @var string
	 */

	private $newEndpoint =
		'http://api.frenzy.me/main/articleprocess';

	private $originalEndpoint =
		'http://api.frenzy.me/main/index';

	/**
	 * The WordPress HTTP transport.
	 *
	 * @var WP_Http
	 */
	private $httpTransport;

	/**
	 * @var string
	 */
	private $username;

	/**
	 * @var string
	 */
	private $apiKey;

	/**
	 * Constructor.
	 *
	 * @param WP_Http $httpTransport
	 */
	public function __construct( WP_Http $httpTransport = null ) {
		$this->httpTransport =
			$httpTransport instanceof WP_Http ? $httpTransport : _wp_http_get_object();
	}

	/**
	 * @param mixed $username
	 */
	public function setUsername( $username ) {
		$this->username = $username;
	}

	/**
	 * @param mixed $apiKey
	 */
	public function setApiKey( $apiKey ) {
		$this->apiKey = $apiKey;
	}

	/**
	 * Get products for an article.
	 *
	 * @param string $articleUrl
	 *
	 * @return int|WP_Error
	 */
	public function getProducts( $articleUrl ) {
		$articleUrl = apply_filters( 'frenzy_article_process_url', $articleUrl );

		if ( ! filter_var( $articleUrl, FILTER_VALIDATE_URL ) ) {
			return new WP_Error( 'invalid_url', 'The article URL is invalid.' );
		}

		$response = $this->notifyArticleProcess( $articleUrl );

		if ( ! is_wp_error( $response ) && isset( $response['Reason'] ) && 'Article already exists in db' === $response['Reason'] ) {
			$response = $this->post(
				$this->originalEndpoint,
				[
					'username' => $this->username,
					'apikey'   => $this->apiKey,
					'url'      => $articleUrl,
				]
			);
		}

		return $response;
	}

	/**
	 * If the response of this endpoint is that it exists in the database, then we query the
	 * original endpoint to get the report; otherwise, we either periodically check back or we do
	 * a full stop (which is determined later in the code below).
	 *
	 * @param   string $articleUrl The URL to process.
	 *
	 * @return  string             The response from the API.
	 */
	public function notifyArticleProcess( $articleUrl ) {
		$articleUrl = apply_filters( 'frenzy_article_process_url', $articleUrl );

		$response = $this->post(
			$this->newEndpoint,
			[
				'username' => $this->username,
				'apikey'   => $this->apiKey,
				'url'      => $articleUrl,
			]
		);

		return $response;
	}

	/**
	 * Performs a POST request to Comment IQ API using the WordPress HTTP transport.
	 *
	 * @param string $url
	 * @param array  $parameters
	 * @param array  $args
	 *
	 * @return array|WP_Error
	 */
	private function post( $url, $parameters, array $args = [] ) {
		if ( empty( $this->username ) || empty( $this->apiKey ) ) {
			return new WP_Error(
				'missing-login-or-password',
				'Make sure you set the username and api key.'
			);
		}

		$args                     = array_merge_recursive( $args, $this->getDefaultArgs() );
		$args['body']             = $parameters;
		$args['body']['username'] = $this->username;
		$args['body']['apikey']   = $this->apiKey;

		return $this->decodeResponse( $this->httpTransport->post( $url, $args ) );
	}

	/**
	 * Get the default arguments for all client requests.
	 *
	 * @return array A set of default arguments needed for the API request.
	 */
	private function getDefaultArgs() {
		return [
			'timeout' => 60,
		];
	}

	/**
	 * Decodes the API response.
	 *
	 * @param mixed $response The response of information that is returned from the server.
	 *
	 * @return array|WP_Error  Will return either the fully validated response of an instance WP_Error.
	 */
	private function decodeResponse( $response ) {
		$response = $this->validateResponse( $response );

		if ( $response instanceof WP_Error ) {
			return $response;
		}

		return $this->validateBody( json_decode( $response['body'], true ) );
	}

	/**
	 * Validates the client response. Returns a WP_Error if an error is found.
	 *
	 * @param mixed $response
	 *
	 * @return array|WP_Error
	 */
	private function validateResponse( $response ) {
		if ( $response instanceof WP_Error ) {
			return $response;
		}

		if ( ! is_array( $response ) ) {
			return new WP_Error( 'invalid_response', 'Response must be an array.' );
		}

		if ( empty( $response['body'] ) ) {
			return new WP_Error( 'invalid_response', 'No body found in the response.' );
		}

		return $response;
	}

	/**
	 * Validates the client response body. Returns a WP_Error if an error is found.
	 *
	 * @param mixed $body
	 *
	 * @return array|WP_Error
	 */
	private function validateBody( $body ) {
		if ( ! is_array( $body ) ) {
			return new WP_Error( 'invalid_json', 'There was an error decoding the response.' );
		}

		if ( empty( $body ) ) {
			return new WP_Error( 'body_empty', 'The response body was empty.' );
		}

		return $body;
	}

	/**
	 * Async call to get the products.
	 *
	 * @param string $articleUrl
	 */
	public function asyncGetProducts( $articleUrl ) {
		$articleUrl = apply_filters( 'frenzy_article_process_url', $articleUrl );

		if ( ! filter_var( $articleUrl, FILTER_VALIDATE_URL ) ) {
			return;
		}

		$this->asyncPost(
			$this->originalEndpoint,
			[
				'username' => $this->username,
				'apikey'   => $this->apiKey,
				'url'      => $articleUrl,
			]
		);
	}

	/**
	 * Performs an async POST request to FrenzyIQ API using CURL and fsock.
	 *
	 * @param string $url
	 * @param array  $parameters
	 *
	 * @return void
	 */
	function asyncPost( $url, $parameters ) {
		$post_params = [];
		foreach ( $parameters as $key => &$val ) {
			if ( is_array( $val ) ) {
				$val = implode( ',', $val );
			}
			$post_params[] = $key . '=' . urlencode( $val );
		}
		$post_string = implode( '' & '', $post_params );

		$parts = parse_url( $url );

		$fp = fsockopen(
			$parts['host'],
			isset( $parts['port'] ) ? $parts['port'] : 80,
			$errno,
			$errstr,
			30
		);

		$out  = 'POST ' . $parts['path'] . " HTTP/1.1\r\n";
		$out .= 'Host: ' . $parts['host'] . "\r\n";
		$out .= "Content-Type: application/x-www-form-urlencoded\r\n";
		$out .= 'Content-Length: ' . strlen( $post_string ) . "\r\n";
		$out .= "Connection: Close\r\n\r\n";
		if ( isset( $post_string ) ) {
			$out .= $post_string;
		}

		fwrite( $fp, $out );
		fclose( $fp );
	}

	/**
	 * Performs a GET request using the WordPress HTTP transport.
	 *
	 * @param string $url   The URL to which the request is being made.
	 * @param array  $query The array of arguments to add to the request.
	 * @param array  $args  The array of additional arguments to add to the query.
	 *
	 * @return array|WP_Error
	 *
	 * @SuppressWarnings(PHPMD.UnusedPrivateMethod)
	 */
	private function get( $url, array $query = [], array $args = [] ) {
		$args = array_merge_recursive( $args, $this->getDefaultArgs() );

		return $this->decodeResponse( $this->httpTransport->get( $this->buildUrl( $url, $query ), $args ) );
	}

	/**
	 * Build an URL with given url and query parameters.
	 *
	 * @param string $url   The URL to which the API is making a request.
	 * @param array  $query The array of arguments used to build the query.
	 *
	 * @return string
	 */
	private function buildUrl( $url, array $query = [] ) {
		return $url . '?' . http_build_query( $query );
	}

	/**
	 * Checks if the given response is considered successful as per the HTTP specification.
	 * This means that the response has a 2xx status code.
	 *
	 * @param array $response The response being validated
	 *
	 * @return bool
	 *
	 * @SuppressWarnings(PHPMD.UnusedPrivateMethod)
	 */
	private function isSuccessful( array $response ) {
		$statusCode = $this->getResponseStatusCode( $response );

		if ( null === $statusCode ) {
			return false;
		}

		return $statusCode >= self::SUCCESSFUL_LOWER && $statusCode < self::SUCCESSFUL_UPPER;
	}

	/**
	 * Extracts the status code from the given response.
	 *
	 * @param array $response The response array with a response code
	 *
	 * @return bool|int
	 */
	private function getResponseStatusCode( array $response ) {
		if ( ! isset( $response['response'] ) || ! is_array( $response['response'] ) ||
			 ! isset( $response['response']['code'] ) ) {
			return false;
		}

		return (int) $response['response']['code'];
	}
}
