<?php

namespace Frenzy\API;

use Frenzy\Helpers\HelperFunctions;

class Indi {
	/**
	 * Minimum payout for Subscriber's to get their earned commission.
	 *
	 * @var int
	 */
	public $minimum_payout = 25;

	public function getCommissionData( $hard_refresh = false ) {
		$commissions = get_transient( 'indi-commissions' );
		if ( empty( $commissions ) || true === $hard_refresh ) {
			$commissions = $this->syncCommissionData();
		}

		if ( empty( $commissions ) ) {
			$commissions = new \stdClass();
		}

		if ( empty( $commissions->PendingCommission ) ) {
			$commissions->PendingCommission = 0;
		}

		if ( empty( $commissions->PayableCommission ) ) {
			$commissions->PayableCommission = 0;
		}

		if ( empty( $commissions->PaidCommission ) ) {
			$commissions->PaidCommission = 0;
		}

		return $commissions;
	}

	public function getPending() {
		$commissions = $this->getCommissionData();

		return '$' . number_format( $commissions->PendingCommission, 2 );
	}

	public function getPayable() {
		$commissions = $this->getCommissionData();

		return '$' . number_format( $commissions->PayableCommission, 2 );
	}

	public function getPaid() {
		$commissions = $this->getCommissionData();

		return '$' . number_format( $commissions->PaidCommission, 2 );
	}

	public function getCommissionDetails() {
		$commissions = get_transient( 'fetch-indi-commissions-details' );
		if ( empty( $commissions ) ) {
			$commissions = $this->syncCommissionDetails();
		}

		return $commissions;
	}

	public function getPayoutHistory() {
		$payouts = get_transient( 'fetch-indi-payouts' );
		if ( empty( $payouts ) ) {
			$payouts = $this->requestPayoutHistory();
		}

		return $payouts;
	}

	protected function syncCommissionData() {
		$body = array();
		if ( $api_key = HelperFunctions::get_fetch_api_key() ) {
			$response = wp_remote_post(
				'http://palmtree.frenzy.ai/api/v1/indi/proxy',
				[
					'headers' => array( 'Content-Type' => 'application/json; charset=utf-8' ),
					'body'    => json_encode(
						[
							'apiKey' => $api_key,
							'method' => 'get',
							'uri'    => '/private/me/commissions/summary',
						]
					),
				]
			);

			$body   = json_decode( wp_remote_retrieve_body( $response ) );
			$status = wp_remote_retrieve_response_code( $response );
			if ( 200 !== $status ) {
				return false;
			}

			set_transient( 'fetch-indi-commissions', $body, HOUR_IN_SECONDS );
		}

		return $body;
	}

	protected function syncCommissionDetails() {
		if ( ! $api_key = HelperFunctions::get_fetch_api_key() ) {
			return false;
		}

		$response = wp_remote_post(
			'http://palmtree.frenzy.ai/api/v1/indi/proxy',
			[
				'headers' => array( 'Content-Type' => 'application/json; charset=utf-8' ),
				'body'    => json_encode(
					[
						'apiKey' => $api_key,
						'method' => 'get',
						'uri'    => '/private/me/commissions/detail'
					]
				),
			]
		);

		$body   = json_decode( wp_remote_retrieve_body( $response ) );
		$status = wp_remote_retrieve_response_code( $response );
		if ( 200 !== $status ) {
			return false;
		}

		set_transient( 'fetch-indi-commissions-details', $body, HOUR_IN_SECONDS );

		return $body;
	}

	/**
	 * Set a subscriber's PayPal email adress in the Indi API.
	 *
	 * Subscribers can register their PayPal email address with Indi so commission payments can be deposited to their PayPal accounts.
	 *
	 * @return bool|array False if email address was not updated or array of the response body.
	 */
	public function updatePaypalEmail() {
		$paypal_email  = HelperFunctions::get_paypal_email();
		$fetch_account = HelperFunctions::get_logged_in_fetch_account_info();
		$indi_email    = $fetch_account['email'];

		if ( ! $api_key = HelperFunctions::get_fetch_api_key() ) {
			return false;
		}

		$response = wp_remote_post(
			'http://palmtree.frenzy.ai/api/v1/indi/proxy',
			[
				'headers' => array( 'Content-Type' => 'application/json; charset=utf-8' ),
				'body'    => json_encode(
					[
						'apiKey' => $api_key,
						'method' => 'post',
						'uri'    => '/manage/users/map',
						'query'  => [
							'client_user_email'  => $indi_email,
							'client_user_name'   => $indi_email,
							'client_user_id'     => get_current_user_id(),
							// looks like this can be anything as long as it's a integer
							'client_user_paypal' => $paypal_email,
						],
					]
				),
			]
		);

		$body   = json_decode( wp_remote_retrieve_body( $response ) );
		$status = wp_remote_retrieve_response_code( $response );
		if ( 200 !== $status ) {
			return false;
		}

		return $body;
	}

	/**
	 * Send a payment request to the Indi API.
	 *
	 * Send a request so a subscriber can have their commissions processed and deposited to their accounts. Until the payment is actually processed by Indi, the subscriber's commissions will remain the same in the Indi system.
	 *
	 * @return bool|string|array False if the Payout request was not successful, false if the payout amount was less than the minimum needed to process a payout request, or array of the response body.
	 */
	public function requestPayout() {
		$commissions = $this->getCommissionData();
		if ( $commissions->PayableCommission < $this->minimum_payout || ! $api_key = HelperFunctions::get_fetch_api_key() ) {
			return false;
		}

		$response = wp_remote_post(
			'http://palmtree.frenzy.ai/api/v1/indi/proxy',
			[
				'headers' => array( 'Content-Type' => 'application/json; charset=utf-8' ),
				'body'    => json_encode(
					[
						'apiKey' => $api_key,
						'method' => 'get',
						'uri'    => '/private/me/commissions/requestpayment',
					]
				),
			]
		);

		$body   = json_decode( wp_remote_retrieve_body( $response ) );
		$status = wp_remote_retrieve_response_code( $response );
		if ( 200 !== $status ) {
			return false;
		}

		return $body;
	}

	/**
	 * Get the Subscriber's payout history from the Indi API
	 *
	 * @return bool|array False if email address was not updated or array of the response body.
	 */
	protected function requestPayoutHistory() {
		if ( ! $api_key = HelperFunctions::get_fetch_api_key() ) {
			return false;
		}

		$response = wp_remote_post(
			'http://palmtree.frenzy.ai/api/v1/indi/proxy',
			[
				'headers' => array( 'Content-Type' => 'application/json; charset=utf-8' ),
				'body'    => json_encode(
					[
						'apiKey' => $api_key,
						'method' => 'get',
						'uri'    => '/private/me/commissions/requestpaymenthistory',
					]
				),
			]
		);

		$body   = json_decode( wp_remote_retrieve_body( $response ) );
		$status = wp_remote_retrieve_response_code( $response );
		if ( 200 !== $status ) {
			return false;
			set_transient( 'fetch-indi-payouts', $body, HOUR_IN_SECONDS );
		}

		return $body;
	}
}
