<?php
/*
 * This file is part of the Frenzy WordPress plugin.
 */

namespace Frenzy\API;

use Frenzy\Helpers\HelperFunctions;

/**
 * Class ShopStyleClient
 *
 * This class is used to interface with the ShopStyle API
 * for adding products to the post admin edit interface, etc.
 *
 * @see     https://shopstylecollective.zendesk.com/hc/en-us/categories/115000180326-ShopStyle-API
 *
 * @package Frenzy\API
 */
class ShopStyleClient {

	/**
	 * The ShopStyle API base endpoint url with placeholders.
	 *
	 * @var string string
	 */
	// private $endpoint_base = 'http://palmtree.frenzy.ai/api/v1/indi/proxy';
	private $endpoint_base = 'https://indi-catalog.azure-api.net';

	/**
	 * The ShopStyle API endpoint url for categories.
	 *
	 * @var string
	 */
	private $endpoint_categories;

	/**
	 * The ShopStyle API endpoint url for brands.
	 *
	 * @var string
	 */
	private $endpoint_brands;

	/**
	 * The ShopStyle API endpoint url for colors.
	 *
	 * @var string
	 */
	private $endpoint_colors;

	/**
	 * The ShopStyle API endpoint url for products.
	 *
	 * @var string
	 */
	private $endpoint_products;

	/**
	 * The ShopStyle API endpoint url for products.
	 *
	 * @var string
	 */
	private $endpoint_retailers;

	/**
	 * The WordPress HTTP transport.
	 *
	 * @var \WP_Http
	 */
	private $httpTransport;

	/**
	 * @var string
	 */
	public $api_key;

	/**
	 * Constructor.
	 *
	 * @param \WP_Http $httpTransport
	 */
	public function __construct( \WP_Http $httpTransport = null ) {
		$this->httpTransport = $httpTransport instanceof \WP_Http ? $httpTransport : _wp_http_get_object();
	}

	public function __get( $name ) {
		// TODO: Implement __get() method.
	}

	public function __set( $name, $value ) {
		// TODO: Implement __set() method.
	}

	/**
	 * Query the ShopStyle API for product data.
	 *
	 * @param array $params The query params.
	 *
	 * @return array|\WP_Error
	 */
	public function get_products( array $params ) {
		$this->set_api_endpoints();
		$api_url  = $this->build_products_api_url( $params );
		$response = $this->query_api( $api_url );

		return $response;
	}

	/**
	 * Set the API endpoints for the ShopStyle API.
	 */
	public function set_api_endpoints() {
		$this->endpoint_brands    = '/catalog/getmanufacturers';
		$this->endpoint_products  = '/catalog/catalogsearch';
		$this->endpoint_retailers = '/catalog/getmerchants';
	}

	/**
	 * Build the products API endpoint url based on params, etc.
	 *
	 * @param array $params The API url parameters.
	 *
	 * @return string
	 */
	private function build_products_api_url( $params ) {
		$info = HelperFunctions::get_logged_in_fetch_account_info();
		if ( empty( $info['indi'] ) || empty( $info['indi']->affiliateId ) ) {
			return false;
		}

		$endpoint_products = $this->endpoint_products . '?page-size=50&sub-id=' . $info['indi']->affiliateId . '&';
		if ( empty( count( $params ) ) ) {
			return $endpoint_products;
		} else {
			if ( ! empty( $params['fl'] ) ) {
				$fl_filters = 'fl=' . $params['fl'] . '&';
				unset( $params['fl'] );
				$url = $endpoint_products . $fl_filters . http_build_query( $params );
			} else {
				$url = $endpoint_products . http_build_query( $params );
			}

			return $url;
		}
	}

	/**
	 * Performs a POST request to Comment IQ API using the WordPress HTTP transport.
	 *
	 * @param string $url The query url.
	 *
	 * @return array|\WP_Error
	 */
	private function query_api( $url ) {
		if ( empty( $this->api_key ) ) {
			return new \WP_Error(
				'missing-login-or-password',
				'Make sure you set the ShopStyle API key in Settings > Frenzy.'
			);
		}

		// return $this->decodeResponse( $this->httpTransport->get( $this->endpoint_base, [
		// 'headers' => array( 'Content-Type' => 'application/json; charset=utf-8' ),
		// 'body'    => json_encode( [
		// 'apiKey' => $this->api_key,
		// 'method' => 'get',
		// 'uri'    => $url,
		// ] ),
		// ] ) );
		return $this->decodeResponse(
			$this->httpTransport->get(
				$this->endpoint_base . $url,
				[
					'headers' => array( 'x-api-key' => 'a1d482d089b0479d9b1fd06ddb2f43ef' ),
					'timeout' => 60,
				]
			)
		);
	}

	/**
	 * Decodes the API response.
	 *
	 * @param mixed $response The response of information that is returned from the server.
	 *
	 * @return array|\WP_Error  Will return either the fully validated response of an instance \WP_Error.
	 */
	private function decodeResponse( $response ) {
		$response = $this->validateResponse( $response );

		if ( $response instanceof \WP_Error ) {
			return $response;
		}

		return $this->validateBody( json_decode( $response['body'], true ) );
	}

	/**
	 * Validates the client response. Returns a \WP_Error if an error is found.
	 *
	 * @param mixed $response
	 *
	 * @return array|\WP_Error
	 */
	private function validateResponse( $response ) {
		if ( $response instanceof \WP_Error ) {
			return $response;
		}

		if ( ! is_array( $response ) ) {
			return new \WP_Error( 'invalid_response', 'Response must be an array.' );
		}

		if ( empty( $response['body'] ) ) {
			return new \WP_Error( 'invalid_response', 'No body found in the response.' );
		}

		return $response;
	}

	/**
	 * Validates the client response body. Returns a \WP_Error if an error is found.
	 *
	 * @param mixed $body
	 *
	 * @return array|\WP_Error
	 */
	private function validateBody( $body ) {
		if ( ! is_array( $body ) ) {
			return new \WP_Error( 'invalid_json', 'There was an error decoding the response.' );
		}

		return $body;
	}

	public function get_user_specific_url( $url ) {
	}

	/**
	 * Set ShopStyle API key
	 *
	 * @param mixed $api_key
	 */
	public function setApiKey( $api_key ) {
		$this->api_key = $api_key;
	}

	public function get_shopstyle_search_filters_data() {
		$this->set_api_endpoints();
		$data              = [];
		$data['brands']    = $this->get_search_filter_data(
			$this->endpoint_brands,
			'manufacturers',
			__( 'Brands', 'frenzy' )
		);
		$data['retailers'] = $this->get_search_filter_data(
			$this->endpoint_retailers,
			'merchants',
			__( 'Retailers', 'frenzy' )
		);

		return $data;
	}

	public function get_search_filter_data( $endpoint, $slug, $label, $cursor = 0, $results = [], $retryCounter = null ) {
		$response     = $this->query_api( $endpoint );
		$retryCounter = is_null( $retryCounter ) ? 1 : $retryCounter;

		if ( ! $response instanceof \WP_Error && ! empty( $response[ $slug ] ) ) {
			$results['id']    = "shopstyle-search-filter-$slug";
			$results['label'] = $label;
			if ( empty( $results['options'] ) ) {
				$results['options'] = [];
			}

			foreach ( $response[ $slug ] as $value ) {
				$results['options'][] = $value;
			}
			natcasesort( $results['options'] );
		} else {
			if ( 10 >= $retryCounter ) {
				$retryCounter ++;
				$results = $this->get_search_filter_data( $endpoint, $slug, $label, $cursor, $results, $retryCounter );
			}
		}

		// if ( ! empty( $response['nextCursor'] ) ) {
		// $results = $this->get_search_filter_data( $endpoint, $slug, $label, $response['nextCursor'], $retryCounter );
		// }

		return $results;
	}

}
