<?php
/**
 * A custom cache built to store information in the Post Metadata table for 12 hours
 */

namespace Frenzy\Cache;

/**
 * A custom cache built to store information in the Post Metadata table for 12 hours. This was built
 * because sites hosted on WordPress.com were subjected to having transients removed at irregular intervals.Acme
 *
 * This resolves the problem by caching the data at the page level for the set amount of time.
 *
 * @since 1.0.0
 */
class FrenzyCache {
	/**
	 * @var int $twelveHours The amount of time for caching the information (12 hours).
	 */
	private $twelveHours = 43200;

	/**
	 * Clears the cache.
	 *
	 * @param int $postId The ID of the post from whose metadata we are clearing information.
	 */
	public function delete( $post_id ) {
		delete_post_meta( $post_id, 'frenzy-duration' );
		delete_post_meta( $post_id, 'frenzy-response' );
	}

	/**
	 * Retrieves information from the cache.
	 *
	 * @param int $postId The ID of the post from whose metadata we are retrieving information.
	 */
	public function get( $post_id ) {
		return get_post_meta( $post_id, 'frenzy-response', true );
	}

	/**
	 * Determines if the page identified by the argument has cached data.
	 *
	 * @param int $postId The ID of the post from whose metadata we look to see if cache data exists.
	 */
	public function has( $post_id ) {
		/**
		 * If it's been more than 10 minutes (or 600 seconds) since it was last pinged,
		 * then we delete the post metadata and then start over.
		 */
		if ( $this->twelveHours < $this->getDuration( $post_id ) ) {
			$this->delete( $post_id );
		}

		return '' !== get_post_meta( $post_id, 'frenzy-response', true );
	}

	/**
	 * Caches data for the specified post using the specified value (which comes from the API).
	 *
	 * @param int          $postId  The ID of the post to which we're associating the information.
	 * @param string|array $value  Information from the Frenzy API we are caching for this page.
	 *
	 * @return bool|int
	 */
	public function set( $post_id, $value ) {
		if ( empty( $value ) ) {
			return false;
		}

		$this->setTime( $post_id );
		return false !== update_post_meta( $post_id, 'frenzy-response', $value );
	}

	/**
	 * Sets the time the data was cached. This time is based on the server on which the site is running.
	 *
	 * @param int $postId  The ID of the post to which we're associating the information.
	 */
	private function setTime( $post_id ) {
		update_post_meta( $post_id, 'frenzy-duration', time() );
	}

	/**
	 * Retrieves the time that the data was cached. This is used to calculate if the cache should be
	 * cleared or not.
	 *
	 * @param int $postId  The ID of the post to which we're associating the information.
	 * @return int         The time, in milliseconds, this data was cached.
	 */
	private function getTime( $post_id ) {
		return (int) get_post_meta( $post_id, 'frenzy-duration', true );
	}

	/**
	 * Calculates how long, in milliseconds, data has been cached for this post as identified by its
	 * argument.
	 *
	 * @param  int $postId The ID of the post for which were testing the age of the cached data.
	 * @return int         The time in milliseconds for how long the data has been cached.
	 */
	private function getDuration( $post_id ) {
		return time() - $this->getTime( $post_id );
	}
}
