<?php
namespace Frenzy\Inc;

use DOMDocument;

/**
 * Taxonomy for Fetch Products.
 */
class FetchTaxonomyFetchProduct extends FetchTaxonomy {

	/**
	 * Name of the taxonomy.
	 *
	 * @var string
	 */
	public $name = 'fetch-product';

	/**
	 * Object types for this taxonomy
	 *
	 * @var array
	 */
	public $object_types;

	/**
	 * Build the taxonomy object.
	 */
	public function __construct() {
		$this->object_types = [ 'attachment' ];

		parent::__construct();
	}

	/**
	 * Creates the taxonomy.
	 */
	public function create_taxonomy() {
		register_taxonomy(
			$this->name,
			$this->object_types,
			[
				'labels'             => [
					'name'                  => __( 'Fetch Products', 'fetch' ),
					'singular_name'         => __( 'Fetch Product', 'fetch' ),
					'search_items'          => __( 'Search Fetch Products', 'fetch' ),
					'popular_items'         => __( 'Popular Fetch Products', 'fetch' ),
					'all_items'             => __( 'All Fetch Products', 'fetch' ),
					'parent_item'           => __( 'Parent Fetch Products', 'fetch' ),
					'parent_item_colon'     => __( 'Parent Fetch Product', 'fetch' ),
					'edit_item'             => __( 'Edit Fetch Product', 'fetch' ),
					'view_item'             => __( 'View Fetch Product', 'fetch' ),
					'update_item'           => __( 'Update Fetch Product', 'fetch' ),
					'add_new_item'          => __( 'Add New Fetch Product', 'fetch' ),
					'new_item_name'         => __( 'New Fetch Product Name', 'fetch' ),
					'add_or_remove_items'   => __( 'Add or remove Fetch Products', 'fetch' ),
					'choose_from_most_used' => __( 'Choose from the most used Fetch Products', 'fetch' ),
					'not_found'             => __( 'No Fetch Products found', 'fetch' ),
					'no_terms'              => __( 'No Fetch Products', 'fetch' ),
					'items_list_navigation' => __( 'Fetch Products list navigation', 'fetch' ),
					'items_list'            => __( 'Fetch Products list', 'fetch' ),
					'back_to_items'         => __( '&larr; Back to Fetch Products', 'fetch' ),
					'menu_name'             => __( 'Fetch Products', 'fetch' ),
					'name_admin_bar'        => __( 'Fetch Products', 'fetch' ),
				],
				'public'             => true,
				'publicly_queryable' => false,
				'show_admin_column'  => true,
				'show_in_nav_menus'  => true,
				'hierarchical'       => true,
				'sort'               => true,
				'show_in_rest'       => true,
				'query_var'          => $this->name,
				'rewrite'            => false,
			]
		);
	}

	// Check if current product exists already. Create if not.
	public static function create( $product, $post_id = null, $attachment_ids = [] ) {
		$term = term_exists( $product->name, 'fetch-product' );

		if ( ! $term ) {
			$term = wp_insert_term(
				$product->name,
				'fetch-product',
				[
					'description' => $product->description,
				]
			);
		}

		$term_meta = [
			'Price'                   => $product->price,
			'ImageUrls'               => [ $product->imageUrl ],
			'Source'                  => $product->clickUrl,
			'Brand'                   => $product->details->brand,
			'Retailer'                => $product->retailer->name,
			'Title'                   => $product->name,
			'InStock'                 => $product->details->in_stock,
			'Onsale'                  => $product->details->sale,
			'score'                   => $product->score,
			'ItemID'                  => $product->id,
			'brand_category_group_id' => $product->brand_category_group_id,
			'user_commission'         => $product->details->commission,
		];
		update_term_meta( $term['term_id'], 'fetch_product_data', $term_meta );
		update_term_meta( $term['term_id'], 'is_manual', ( ! empty( $product->is_manual ) ) ? $product->is_manual : false );

		if ( isset( $product->image_ref->id ) && ! isset( $attachment_ids[ $product->image_ref->id ] ) ) {
			$attachment_id                             = self::get_attachment_id_by_url( $product->image_ref->url );
			$attachment_ids[ $product->image_ref->id ] = $attachment_id;
			wp_set_post_terms( $attachment_ids[ $product->image_ref->id ], [ $term['term_id'] ], 'fetch-product', true );
		}

		return $term['term_id'];
	}

	public static function get_attachment_id_by_url( $url ) {
		add_filter( 'remove_placeholder_escape', '__return_true' );
		$clean_image_url      = self::clean_image_url( $url );
		$database_prepped_url = str_replace( '&', '&amp;', $clean_image_url );
		$attachment_id        = attachment_url_to_postid( $database_prepped_url );
		if ( empty( $attachment_id ) ) {
			global $wpdb;
			$sql           = $wpdb->prepare(
				"SELECT ID FROM $wpdb->posts WHERE guid = %s",
				$database_prepped_url
			);
			$attachment_id = $wpdb->get_var( $sql );
		}

		remove_filter( 'remove_placeholder_escape', '__return_true' );
		return $attachment_id;
	}

	public static function get_manually_added_products( $post_id ) {
		$term_query = new \WP_Term_Query();

		$results  = $term_query->query(
			[
				'taxonomy'   => 'fetch-product',
				'object_ids' => [ (int) $post_id ] + self::get_images_for_post( $post_id ),
				'meta_key'   => 'is_manual',
				'meta_value' => '1',
			]
		);
		$products = [];

		foreach ( $results as $result ) {
			$product                        = self::get_product( $result->term_id );
			$products[ $product['ItemID'] ] = $product['id'];
		}

		return $products;
	}

	public static function get_products_attached_to_post( $post_id ) {
		$term_query = new \WP_Term_Query();

		return $term_query->query(
			[
				'taxonomy'   => 'fetch-product',
				'object_ids' => [ (int) $post_id ] + self::get_images_for_post( $post_id ),
				'orderby'    => 'term_order',
			]
		);
	}

	public static function get_images_for_post( $post_id, $post_content = '' ) {
		if ( empty( $post_content ) ) {
			$post_content = get_post_field( 'post_content', $post_id );
		}

		if ( empty( $post_content ) ) {
			return [];
		}

		$document = new DOMDocument();
		libxml_use_internal_errors( true );
		$document->loadHTML( utf8_decode( $post_content ) );

		$images         = $document->getElementsByTagName( 'img' );
		$attachment_ids = [];
		foreach ( $images as $image ) {
			$src           = self::clean_image_url( $image->getAttribute( 'src' ) );
			$attachment_id = self::get_attachment_id_by_url( $src );
			if ( ! $attachment_id ) {
				$object = array(
					'post_title'     => $src,
					'post_content'   => $src,
					'post_parent'    => $post_id,
					'post_mime_type' => self::fetch_get_mime_type( $src ),
					'guid'           => $src,
				);

				$attachment_id = wp_insert_attachment( $object, $src );
			}

			$attachment_ids[ $src ] = $attachment_id;
		}

		// Include the featured image if it exists.
		$featured_image_url = get_the_post_thumbnail_url( $post_id );
		if ( ! empty( $featured_image_url ) ) {
			$id             = self::get_attachment_id_by_url( $featured_image_url );
			$attachment_ids = [ $featured_image_url => $id ] + $attachment_ids;
		}

		return $attachment_ids;
	}

	public static function get_images_uploaded_to_post( $post_id ) {
		$query = new \WP_Query(
			array(
				'post_parent'    => $post_id,
				'post_type'      => 'attachment',
				'posts_per_page' => 99,
				'post_status'    => 'inherit',
			)
		);

		$attachment_ids = [];
		foreach ( $query->posts as $post ) {
			$attachment_ids[] = $post->ID;
		}

		return $attachment_ids;
	}

	public static function get_product( $term_id ) {
		$term                   = get_term( $term_id );
		$product                = get_term_meta( $term_id, 'fetch_product_data', true );
		$product['id']          = $term_id;
		$product['description'] = $term->description;
		$manual                 = get_term_meta( $term_id, 'is_manual', true );
		$product['is_manual']   = ( empty( $manual ) ) ? 0 : $manual;

		return $product;
	}

	public static function get_products_for_image( $image_id ) {
		$products = wp_get_object_terms( $image_id, 'fetch-product', [ 'orderby' => 'term_order' ] );
		$prods    = [];

		foreach ( (array) $products as $product ) {
			if ( empty( $product ) ) {
				continue;
			}
			$prods[] = self::get_product( $product->term_id );
		}

		return $prods;
	}

	// Sometimes WP image URLs include sizing appended. We need to strip that if we are going to find the
	// attachment ID.
	public static function clean_image_url( $url ) {
		$url   = wp_unslash( str_replace( '"', '', $url ) );
		$re    = '/-\d+[Xx]\d+\./';
		$subst = '.';
		return preg_replace( $re, $subst, $url, 1 );
	}

	/**
	 * Get mime type from file path if allow_url_fopen is off.
	 *
	 * @param string $src Image url.
	 */
	public static function fetch_get_mime_type( $src ) {

		$mime_src = $src;
		if ( ! ini_get( 'allow_url_fopen' ) ) {
			$mime_src = get_home_path() . wp_make_link_relative( $src );
		}

		$mime = wp_get_image_mime( $mime_src );

		return $mime;
	}
}
