<?php

/**
 * This file is part of the Frenzy WordPress plugin.
 * (c) Pressware, LLC <support@pressware.co>
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * @package Frenzy
 */

namespace Frenzy\Subscriber;

use Frenzy\API\FrenzyClient;
use Frenzy\Cache\FrenzyCache;
use Frenzy\Helpers\HelperFunctions;
use Frenzy\Inc\FetchTaxonomyFetchProduct;
use Frenzy\PluginAPI\HookSubscriberInterface;

/**
 * Class AjaxSubscriber
 *
 * @package Frenzy\Subscriber
 */
class AjaxSubscriber implements HookSubscriberInterface {
	/**
	 * Frenzy API client.
	 *
	 * @var FrenzyClient
	 */
	private $client;

	/**
	 * The plugin cache.
	 *
	 * @var FrenzyCache
	 */
	private $cache;

	/**
	 * AjaxSubscriber constructor.
	 *
	 * @param FrenzyClient $client The API client responsible for talking to the Frenzy API.
	 * @param FrenzyCache  $cache  The cache responsible for caching the response data to the post metadata table.
	 */
	public function __construct( FrenzyClient $client, FrenzyCache $cache ) {
		$this->client = $client;
		$this->cache  = $cache;
	}

	/**
	 * Defines the Ajax hooks for the products to use when the JavaScript makes the request.
	 */
	public static function getHooks() {
		return [
			'wp_ajax_nopriv_frenzy_get_products'  => 'get_products',
			'wp_ajax_frenzy_get_products'         => 'get_products',
			'wp_ajax_fetch_create_account'        => 'create_account',
			'wp_ajax_fetch_edit_account'          => 'edit_account',
			'wp_ajax_fetch_forgot_password'       => 'forgot_password',
			'wp_ajax_fetch_login'                 => 'login',
			'wp_ajax_get_manually_added_products' => 'ajax_get_manually_added_products',
		];
	}

	/**
	 * Checks the cache for the product data. If it's not present, then queries the API and returns the result to
	 * the JavaScript file requesting the information.
	 */
	public function get_products() {
		// Sanitize the incoming URL.
		$url       = sanitize_text_field( filter_input( INPUT_POST, 'url' ) );
		$post_id   = sanitize_text_field( filter_input( INPUT_POST, 'postId' ) );
		$api_error = false;

		if ( $this->cache->has( $post_id ) ) {
			wp_send_json_success( $this->cache->get( $post_id ) );
		}

		// Otherwise, get the response for the incoming URL.
		$response = $this->client->getProducts( $url );

		if ( $response instanceof \WP_Error ) {
			$api_error = $response->get_error_message();
		}

		if ( 'Article already being processed' === $response['Status'] ) {
			wp_send_json_success( $response );
			exit;
		}

		// If we have an error, then we need to return that to the browser.
		if ( ! empty( $response['Status'] ) && 'Error' === $response['Status'] ) {
			$api_error = $response;
		}

		if ( $api_error ) {
			// Return manual products even if the API returned an error.
			$manual_products = $this->get_added_shopstyle_products( $post_id );
			if ( ! empty( $manual_products ) ) {
				wp_send_json_success( $manual_products );
				exit;
			}

			wp_send_json_error( $api_error );
			exit;
		}

		// Set the cache on a valid response then send the result of the cache to the browser.
		$this->update_products( $post_id, $response );
		wp_send_json_success( $this->cache->get( $post_id ) );
	}

	/**
	 * Product reordering based on a array found in post meta
	 * 'frenzy-response-product-order'. This array is used to
	 * reorder products when the cache is renewed as well. Newly
	 * added products not contained in the ordering array are
	 * added to the end of the product list.
	 *
	 * @param array $products        The products to be reordered.
	 * @param array $reordering_data The array the determines the ordering.
	 *
	 * @return array The reordered products.
	 */
	private function reorderProducts( $products = [], $reordering_data = [] ) {
		if ( empty( $reordering_data ) || empty( $products ) ) {
			return $products;
		}

		// After ordering data post meta is updated, let's get the product data,
		// re-order it based on the $reordering_data and re-save it in the database.
		$reordered_products = [];
		$added_products     = [];
		foreach ( $products as $key => $data ) {
			foreach ( $reordering_data as $reordering_data_key => $product_id ) {
				if ( $product_id == $data['ItemID'] ) {
					$reordered_products[ $reordering_data_key ] = $data;
				} else {
					// If we didn't have a match & the product wasn't in the
					// ordering array then that product has been added since
					// storing the ordering array, so well account for those
					// products and add them to the end of the array before
					// returning the $reordered_product_data.
					if ( ! in_array( $data['ItemID'], $reordering_data, false ) ) {
						$added_products[ $key ] = $data;
					}
				}
			}
		}
		if ( ! empty( $reordered_products ) ) {
			ksort( $reordered_products );
			// If new products have been added since the previous re-ordering, well merge them into the end of the array.
			$reordered_products = ! empty( $added_products ) ? array_merge( $reordered_products, $added_products ) : $reordered_products;
		} else {
			$reordered_products = $products;
		}

		return $reordered_products;
	}

	/**
	 * Remove deleted products from Frenzy API call if present.
	 *
	 * @param int   $post_id  The product id.
	 * @param array $products The products array from the API.
	 *
	 * @return array
	 */
	private function remove_deleted_products( $post_id, $products ) {
		$deleted_products = $this->get_deleted_products( $post_id );
		if ( ! is_array( $deleted_products ) || empty( $deleted_products ) ) {
			return $products;
		}
		foreach ( $products as $key => $product ) {
			if ( in_array( intval( $product['ItemID'] ), $deleted_products, true ) ) {
				unset( $products[ $key ] );
			}
		}

		return $products;
	}

	/**
	 * Get deleted products from post meta.
	 *
	 * @param int $post_id The post id.
	 *
	 * @return int|bool meta_id if the meta doesn't exist, otherwise returns true on success and false on failure.
	 */
	private function get_deleted_products( $post_id ) {
		return get_post_meta( $post_id, 'frenzy-deleted-products', true );
	}

	public function create_account() {
		$account_details = [
			'email'      => sanitize_email( wp_unslash( $_POST['email-address'] ) ),
			'name'       => sanitize_text_field( wp_unslash( $_POST['name'] ) ),
			'password'   => sanitize_text_field( wp_unslash( $_POST['password'] ) ),
			'inviteCode' => sanitize_text_field( wp_unslash( $_POST['invite-code'] ) ),
			'domain'     => home_url(),
		];

		$response = wp_remote_post(
			'http://palmtree.frenzy.ai/api/v1/user/signup',
			[
				'headers' => array( 'Content-Type' => 'application/json; charset=utf-8' ),
				'body'    => json_encode( $account_details ),
				'timeout' => 30,
			]
		);

		$body   = json_decode( wp_remote_retrieve_body( $response ) );
		$status = wp_remote_retrieve_response_code( $response );
		if ( 201 !== $status ) {
			wp_send_json_error( $body );
			wp_die();
		}

		HelperFunctions::save_fetch_account_info(
			[
				'email'  => $body->data->email,
				'name'   => $body->data->name,
				'apiKey' => $body->data->apiKey,
				'indi'   => $body->data->indi,
			]
		);
		wp_send_json_success( $body->data );
		wp_die();
	}

	public function edit_account() {
		$account_details = [
			'email'           => sanitize_email( wp_unslash( $_POST['email-address'] ) ),
			'name'            => sanitize_text_field( wp_unslash( $_POST['name'] ) ),
			'currentPassword' => sanitize_text_field( wp_unslash( $_POST['current_password'] ) ),
			'newPassword'     => sanitize_text_field( wp_unslash( $_POST['new_password'] ) ),
			'apiKey'          => HelperFunctions::get_fetch_api_key(),
		];

		if ( empty( $account_details['newPassword'] ) ) {
			unset( $account_details['newPassword'] );
		}

		$response = wp_remote_post(
			'http://palmtree.frenzy.ai/api/v1/user/edit',
			[
				'headers' => array(
					'Content-Type' => 'application/json; charset=utf-8',
				),
				'body'    => json_encode( $account_details ),
			]
		);

		$body   = json_decode( wp_remote_retrieve_body( $response ) );
		$status = wp_remote_retrieve_response_code( $response );
		if ( 200 !== $status ) {
			wp_send_json_error( $body );
			wp_die();
		}

		HelperFunctions::save_fetch_account_info(
			[
				'email'  => $body->data->email,
				'name'   => $body->data->name,
				'apiKey' => $body->data->apiKey,
				'indi'   => $body->data->indi,
			]
		);
		wp_send_json_success( $body->data );
		wp_die();
	}

	public function forgot_password() {
		$account_details = [
			'email'      => sanitize_email( wp_unslash( $_POST['email-address'] ) ),
			'return_url' => admin_url( 'options-general.php?page=fetch-settings-submenu&tab=accounts' ),
		];

		$response = wp_remote_post(
			'http://palmtree.frenzy.ai/api/v1/password/forgot',
			[
				'headers' => array(
					'Content-Type' => 'application/json; charset=utf-8',
				),
				'body'    => json_encode( $account_details ),
			]
		);

		$body   = json_decode( wp_remote_retrieve_body( $response ) );
		$status = wp_remote_retrieve_response_code( $response );

		error_log( print_r( $status, true ) );
		error_log( print_r( $body, true ) );

		if ( 200 !== $status ) {
			wp_send_json_error( $body );
			wp_die();
		}

		wp_send_json_success( $body->data );
		wp_die();
	}

	public function reset_password() {
		$account_details = [
			'email'    => sanitize_email( $_POST['email'] ),
			'password' => sanitize_text_field( wp_unslash( $_POST['password'] ) ),
			'token'    => sanitize_text_field( wp_unslash( $_POST['token'] ) ),
		];

		$response = wp_remote_post(
			'http://palmtree.frenzy.ai/api/v1/password/change',
			[
				'headers' => array(
					'Content-Type' => 'application/json; charset=utf-8',
				),
				'body'    => json_encode( $account_details ),
			]
		);

		$body   = json_decode( wp_remote_retrieve_body( $response ) );
		$status = wp_remote_retrieve_response_code( $response );

		if ( 200 !== $status ) {
			wp_send_json_error( $body );
			wp_die();
		}

		wp_send_json_success( $body->data );
		wp_die();
	}

	public function login() {
		$response = wp_remote_post(
			'http://palmtree.frenzy.ai/api/v1/user/login',
			[
				'headers' => array( 'Content-Type' => 'application/json; charset=utf-8' ),
				'body'    => json_encode(
					[
						'email'    => sanitize_email( $_POST['email-address'] ),
						'password' => sanitize_text_field( wp_unslash( $_POST['password'] ) ),
					]
				),
			]
		);

		$body   = json_decode( wp_remote_retrieve_body( $response ) );
		$status = wp_remote_retrieve_response_code( $response );
		if ( 200 !== $status ) {
			wp_send_json_error( $body );
			wp_die();
		}

		HelperFunctions::save_fetch_account_info(
			[
				'email'  => $body->data->email,
				'name'   => $body->data->name,
				'apiKey' => $body->data->apiKey,
				'indi'   => $body->data->indi,
			]
		);
		wp_send_json_success( [ 'redirect_to' => admin_url( 'options-general.php?page=fetch-settings-submenu&tab=accounts' ) ] );
		wp_die();
	}

	protected function save_account_info_from_api( $response ) {
	}

	public function ajax_get_manually_added_products() {
		wp_send_json_success( FetchTaxonomyFetchProduct::get_manually_added_products( $_REQUEST['post_id'] ) );
	}
}
