<?php
/**
 * Saves the Frenzy settings from the post meta box.
 */

namespace Frenzy\Subscriber;

use Frenzy\PluginAPI\AbstractSubscriber;
use Frenzy\PluginAPI\HookSubscriberInterface;
use Frenzy\API\Indi;

/**
 * Saves the Frenzy PayPal settings.
 *
 * @SuppressWarnings(PHPMD)
 */
class SavePaypalSubscriber extends AbstractSubscriber implements HookSubscriberInterface {

	/**
	 * @var string The path to the URL on disk.
	 */
	private $pluginPath;

	/**
	 * Initializes the plugin by setting the plugin path.
	 *
	 * @param string $pluginPath The path to the plugin on disk.
	 */
	public function __construct( $pluginPath ) {
		$this->pluginPath = $pluginPath;
	}

	/**
	 * Defines the hooks to which this class subscribes.
	 */
	public static function getHooks() {
		return array(
			'admin_init' => 'maybeSave',
		);
	}

	/**
	 * Determines if the user can save information and, if so, saves them; otherwise, serializes an error.
	 */
	public function maybeSave() {
		if ( ! isset( $_POST['fetch-save-paypal-email-settings'] ) && ! isset( $_POST['fetch-save-paypal-payout-settings'] ) ) {
			return;
		}

		if ( isset( $_POST['fetch-save-paypal-email-settings'] ) ) {
			if ( ! $this->userCanSave( 'fetch-wordpress-plugin', 'fetch-wordpress-plugin-paypal-email' ) ) {
				update_option( 'fetch-paypal-settings-status', 'error' );
			} elseif ( $this->userCanSave( 'fetch-wordpress-plugin', 'fetch-wordpress-plugin-paypal-email' ) ) {
				$this->savePayPalSettings();
			}
		}
		if ( isset( $_POST['fetch-save-paypal-payout-settings'] ) ) {
			if ( ! $this->userCanSave( 'fetch-wordpress-plugin', 'fetch-wordpress-plugin-paypal-request-payout' ) ) {
				update_option( 'fetch-paypal-settings-status', 'error' );
			} elseif ( $this->userCanSave( 'fetch-wordpress-plugin', 'fetch-wordpress-plugin-paypal-request-payout' ) ) {
				$this->requestPayout();
			}
		}

		wp_safe_redirect(
			$_REQUEST['_wp_http_referer'],
			301
		);
		exit;
	}

	/**
	 * Saves the subscriber's PayPal email address.
	 *
	 * @access private
	 */
	private function savePayPalSettings() {
		$indi = new Indi();
		update_option( 'fetch-paypal-email', sanitize_text_field( wp_unslash( $_POST['fetch-paypal-email'] ) ) );

		if ( false !== $indi->updatePaypalEmail() ) {
			update_option( 'fetch-paypal-settings-status', 'success' );
		} else {
			update_option( 'fetch-paypal-settings-status', 'error' );
		}
	}

	/**
	 * Send a request to the Indi API to pay out a Subscriber their earned commission
	 *
	 * @return void
	 *
	 * @access private
	 */
	private function requestPayout() {
		$indi = new Indi();
		if ( true === $indi->requestPayout() ) {
			update_option( 'fetch-paypal-payout-status', 'success' );
			$payout_history = get_option( 'fetch-payouts', [] );
			$today_timestamp = time();
			$payout_amount = $indi->getPayable();
			$payout_history[] = [ 'payout' => $payout_amount, 'time' => $today_timestamp ];

			if ( ! empty( $payout_history ) ) {
				update_option( 'fetch-payouts', $payout_history );
			}
		} else {
			update_option( 'fetch-paypal-payout-status', 'error' );
		}
	}

	/**
	 * Determines if the user has permission to save information.
	 *
	 * @param string $nonce  The security value used to determine if the user can save information.
	 * @param string $action The action associated with the nonce.
	 *
	 * @return bool Whether or not the user can save information.
	 *
	 * @access private
	 */
	private function userCanSave( $nonce, $action ) {
		$isNonceSet   = isset( $_POST[ $nonce ] );
		$isValidNonce = false;

		if ( $isNonceSet ) {
			$isValidNonce = wp_verify_nonce( $_POST[ $nonce ], $action );
		}

		return ( $isNonceSet && $isValidNonce );
	}
}
