<?php
/**
 * Saves the Frenzy settings from the post meta box.
 */

namespace Frenzy\Subscriber;

use Frenzy\PluginAPI\AbstractSubscriber;
use Frenzy\PluginAPI\HookSubscriberInterface;

/**
 * Saves the Frenzy settings.
 *
 * @SuppressWarnings(PHPMD)
 */
class SaveSettingsSubscriber extends AbstractSubscriber implements HookSubscriberInterface {

	/**
	 * @var string The path to the URL on disk.
	 */
	private $pluginPath;

	/**
	 * Initializes the plugin by setting the plugin path.
	 *
	 * @param string $pluginPath The path to the plugin on disk.
	 */
	public function __construct( $pluginPath ) {
		$this->pluginPath = $pluginPath;
	}

	/**
	 * Defines the hooks to which this class subscribes.
	 */
	public static function getHooks() {
		 return array(
			 'admin_init' => 'maybeSave',
		 );
	}

	/**
	 * Determines if the user can save information and, if so, saves them; otherwise, serializes an error.
	 */
	public function maybeSave() {
		if ( ! isset( $_POST['frenzy-save-settings'] ) ) {
			return;
		}

		if ( ! $this->userCanSave( 'frenzy-wordpress-plugin', 'frenzy-wordpress-plugin-settings' ) ) {
			update_option( 'frenzy-settings-status', 'error' );
		}

		$this->saveSettings();

		wp_safe_redirect(
			$_REQUEST['_wp_http_referer'],
			301
		);
		exit;
	}

	/**
	 * Saves the username and API key to the database.
	 *
	 * @access private
	 */
	private function saveSettings() {
		update_option( 'frenzy-settings-status', 'success' );
		update_option( 'fetch-show-home-feed', sanitize_text_field( wp_unslash( $_POST['fetch-show-home-feed'] ) ) );
		update_option( 'fetch-position-horizontal', sanitize_text_field( wp_unslash( $_POST['fetch-position-horizontal'] ) ) );
		update_option( 'fetch-position-vertical', sanitize_text_field( wp_unslash( $_POST['fetch-position-vertical'] ) ) );
		update_option( 'fetch-shop-btn-show-always', sanitize_text_field( wp_unslash( $_POST['fetch-shop-btn-show-always'] ) ) );
	}

	/**
	 * Determines if the user has permission to save information.
	 *
	 * @param string $nonce  The security value used to determine if the user can save information.
	 * @param string $action The action associated with the nonce.
	 *
	 * @return bool Whether or not the user can save information.
	 *
	 * @access private
	 */
	private function userCanSave( $nonce, $action ) {
		$isNonceSet   = isset( $_POST[ $nonce ] );
		$isValidNonce = false;

		if ( $isNonceSet ) {
			$isValidNonce = wp_verify_nonce( $_POST[ $nonce ], $action );
		}

		return ( $isNonceSet && $isValidNonce );
	}
}
