<?php

namespace Frenzy\Subscriber;

use Frenzy\Cache\FrenzyCache;
use Frenzy\Helpers\HelperFunctions;
use Frenzy\Inc\FetchTaxonomyFetchProduct;
use Frenzy\PluginAPI\AbstractSubscriber;
use Frenzy\PluginAPI\HookSubscriberInterface;

/**
 * Adds the administration stylesheets.
 *
 * @since 1.0.0
 */
class ScriptAssetSubscriber extends AbstractSubscriber implements HookSubscriberInterface {
	/**
	 * The plugin url.
	 *
	 * @var string $plugin_url The URL of the on disk.
	 */
	private $plugin_url;

	/**
	 * The current WP screen.
	 *
	 * @var string $current_screen The current WP screen.
	 */
	private $current_screen;

	/**
	 * Constructor.
	 *
	 * @param string $plugin_url The plugin url.
	 */
	public function __construct( $plugin_url ) {
		$this->plugin_url = $plugin_url;
	}

	/**
	 * {@inheritdoc}
	 */
	public static function getHooks() {
		return [
			'admin_enqueue_scripts' => 'enqueue',
		];
	}

	/**
	 * If we're not a settings screen, then adds the admin JavaScript.
	 */
	public function enqueue() {
		$this->current_screen = get_current_screen();

		if ( true === $this->is_settings_screen() || true === $this->is_post_edit_screen() ) {
			wp_register_script(
				'frenzy-admin',
				$this->plugin_url . 'assets/scripts/dev/admin.js',
				[ 'jquery' ],
				filemtime( dirname( dirname( dirname( __FILE__ ) ) ) . '/assets/scripts/dev/admin.js' )
			);

			wp_localize_script(
				'frenzy-admin',
				'frenzyWp',
				[
					'ajaxUrl'              => admin_url( 'admin-ajax.php' ),
					'loadingSpinnerUrl'    => esc_url( FRENZY_ASSETS_URL . 'assets/images/ajax-loader.gif' ),
					'fetchProductsTextUrl' => esc_url( FRENZY_ASSETS_URL . 'assets/images/fetch-white.png' ),
					'fetchLogoUrl'         => esc_url( FRENZY_ASSETS_URL . 'assets/images/fetch-logo.png' ),
				]
			);
			wp_enqueue_script( 'frenzy-admin' );

			wp_register_script(
				'jquery-datatables',
				$this->plugin_url . 'assets/scripts/jquery.dataTables.js',
				[ 'jquery' ],
				filemtime( dirname( dirname( dirname( __FILE__ ) ) ) . '/assets/scripts/jquery.dataTables.js' )
			);
			wp_enqueue_script( 'jquery-datatables' );

			wp_register_style(
				'jquery-datatables',
				$this->plugin_url . 'assets/styles/css/jquery.dataTables.css',
				[],
				filemtime( dirname( dirname( dirname( __FILE__ ) ) ) . '/assets/styles/css/jquery.dataTables.css' )
			);
		}

		if ( true === $this->is_post_edit_screen() ) {
			wp_enqueue_script(
				'frenzy-products-chosen',
				$this->plugin_url . 'assets/scripts/chosen.jquery.js',
				[ 'jquery' ],
				filemtime( dirname( dirname( dirname( __FILE__ ) ) ) . '/assets/scripts/chosen.jquery.js' ),
				true
			);
			wp_enqueue_script(
				'frenzy-sortable-products',
				$this->plugin_url . 'assets/scripts/dev/admin-sortable-products.js',
				[ 'jquery', 'jquery-ui-core', 'jquery-ui-draggable', 'jquery-ui-sortable', 'frenzy-products-chosen' ],
				filemtime( dirname( dirname( dirname( __FILE__ ) ) ) . '/assets/scripts/dev/admin-sortable-products.js' ),
				true
			);
			global $post;
			$cache                   = new FrenzyCache();
			$frenzy_wp_sortable_vars = [
				'ajaxUrl'                       => admin_url( 'admin-ajax.php' ),
				'reoderingAction'               => 'frenzy_shopsimilar_product_reordering',
				'postID'                        => $post->ID,
				'productRemovalAction'          => 'frenzy_shopsimilar_product_removal',
				'addProductAction'              => 'frenzy_shopstyle_product_addition',
				'addProductFromShopStyleAction' => 'frenzy_shopstyle_add_product_from_modal',
				'manuallyAddedProducts'         => FetchTaxonomyFetchProduct::get_manually_added_products( $post->ID ),
				'addProductFromShopStyleNonce'  => wp_create_nonce( 'frenzy_shopstyle_add_product_from_modal_nonce' ),
				'shopStyleProductItemHtml'      =>
					'<div class="shopstyle-product-item">
						<img class="photo" src="" alt="">
						<p class="brand"></p>
						<p class="product"></p>
						<p class="price"></p>
						<div class="fetch-product-item-actions">
							<button class="button fetch-manual-product-view-details">View Product Details</button>
						</div>
					</div>',
				'buttonProductAddedText'        => __( 'Product Already Added', 'frenzy' ),
				'getShopstyleApiParamMap'       => HelperFunctions::get_shopstyle_api_param_map(),
				'getPriceFilterOptions'         => HelperFunctions::get_price_filter_options(),
				'isFetchLoggedIn'               => HelperFunctions::is_fetch_logged_in(),
				'settingsUrl'                   => admin_url( 'options-general.php?page=fetch-settings-submenu' ),
			];

			wp_localize_script( 'frenzy-sortable-products', 'frenzyWpSortable', $frenzy_wp_sortable_vars );

			wp_localize_script( 'frenzy-admin', 'frenzyPostSettings', HelperFunctions::get_fetch_shop_settings( $post->ID ) );
		}
	}



	/**
	 * Is this the settings screen?
	 *
	 * @access private
	 * @return bool True if we are on the Frenzy settings page.
	 */
	private function is_settings_screen() {
		return 'settings_page_fetch-settings-submenu' === $this->current_screen->id;
	}

	/**
	 * Is this the post edit screen?
	 *
	 * @access private
	 * @return bool Whether or not we are on the Post screen.
	 */
	private function is_post_edit_screen() {
		return 'post' === $this->current_screen->id;
	}
}
