<?php

/**
 * This file is part of the Frenzy WordPress plugin.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * @package Frenzy
 */

namespace Frenzy\Subscriber;

use Frenzy\API\ShopStyleClient;
use Frenzy\Cache\FrenzyCache;
use Frenzy\PluginAPI\HookSubscriberInterface;

class ShopStyleAjaxSubscriber implements HookSubscriberInterface {
	/**
	 * API client.
	 *
	 * @var ShopStyleClient
	 */
	private $client;

	/**
	 * The plugin cache.
	 *
	 * @var FrenzyCache
	 */
	private $cache;

	/**
	 * ShopStyleAjaxSubscriber constructor.
	 *
	 * @param ShopStyleClient $client The API client responsible for talking to the ShopStyle API.
	 * @param FrenzyCache     $cache  The cache responsible for caching the response data to the post metadata table.
	 */
	public function __construct( ShopStyleClient $client, FrenzyCache $cache ) {
		$this->client = $client;
		$this->cache  = $cache;
	}

	/**
	 * Defines the Ajax hooks for the products to use when the JavaScript makes the request.
	 */
	public static function getHooks() {
		return [
			'wp_ajax_frenzy_shopstyle_product_addition' => 'get_products',
			'wp_ajax_frenzy_shopstyle_add_product_from_modal' => 'add_single_shopstyle_product',
		];
	}

	/**
	 * Get ShopStyle products from the API.
	 */
	public function get_products() {
		if ( ! wp_verify_nonce( $_POST['addProductNonce'], 'frenzy_shopstyle_product_addition' ) ) { // @codingStandardsIgnoreLine
			die( 'failed nonce verification' );
		}

		// Process any search or filter params.
		$params = [];
		if ( ! empty( $_POST['queryParams'] ) ) {
			foreach ( $_POST['queryParams'] as $key => $param ) {
				if ( 'fl' === $key && is_array( $param ) ) {
					$params[ sanitize_text_field( $key ) ] = sanitize_text_field( implode( '&fl=', $param ) );
				} else {
					$params[ sanitize_text_field( $key ) ] = sanitize_text_field( $param );
				}
			}
		}

		// Sanitize the incoming data.
		$post_id = sanitize_text_field( filter_input( INPUT_POST, 'postId' ) );

		if ( ! empty( $post_id ) ) {
			$params['post_id'] = $post_id;
		}
		$response = $this->client->get_products( $params );

		// Set the cache on a valid response then send the result of the cache to the browser.
		if ( $response instanceof \WP_Error ) {
			wp_send_json_error();
		} else {
			wp_send_json_success( $response );
		}
	}

	/**
	 * Add product from ShopStyle API query.
	 */
	public function add_single_shopstyle_product() {
		if ( ! wp_verify_nonce( $_POST['addProductFromShopStyleNonce'], 'frenzy_shopstyle_add_product_from_modal_nonce' ) ) { // @codingStandardsIgnoreLine
			die( 'failed nonce verification' );
		}

		if ( empty( $_POST['product'] ) ) {
			wp_send_json_error( 'No product in AJAX post data', 'no-product' );
		}

		$product = $this->map_product_data( $_POST['product'] );

		if ( ! empty( $product ) ) {
			$post_id  = intval( $_POST['postID'] );
			$products = $this->cache->get( $post_id );
			if ( empty( $products ) ) {
				$products = [];
			}
			$this->update_shopsimilar_only_product_additions( $post_id, $product );
			$products[] = $product;
			if ( true === $this->cache->set( $post_id, $products ) ) {
				wp_send_json_success( $product );
			} else {
				wp_send_json_error( 'Product could not be added to the database' );
			}
		} else {
			wp_send_json_error( 'Product could not be obtained from the API' );
		}
	}

	/**
	 * Take the data received for the product being added
	 * to the sortable array and map it's structure to those
	 * used by products obtained from the Frenzy array.
	 *
	 * Should end up in the following similar format...
	 *
	 * [
	 *    'ItemID'             => '15406',
	 *    'ItemGuID'           => '5abdcfa3e09ed',
	 *    'Brand'              => 'Rag & Bone',
	 *    'Title'              => 'rag & bone/JEAN Nico Stretch Denim Jacket',
	 *    'Price'              => '$295',
	 *    'Source'             =>
	 *    'https://api.shopstyle.com/action/apiVisitRetailer?id=712028835&pid=uid7444-33521462-78',
	 *    'Description'        => 'Vintage-inspired snap denim jacket. Snap-button front. Long sleeves with snap-button
	 *    cuffs. Chest snap-button pockets. Waist welt pockets. About 21" long from shoulder to hem. Cotton/elastane.
	 *    Machine wash. Made in USA. Model shown is 5\'10" (177cm) wearing US size Small. Worn Indigo',
	 *    'Materials'          =>
	 *        [
	 *            0 => 'Cotton',
	 *            1 => 'Denim',
	 *            2 => 'Synthetic',
	 *        ],
	 *    'ImageUrls'          =>
	 *        [
	 *            0 => 'https://img.shopstyle-cdn.com/pim/6d/6d/6d6d5f201c2a149f6b29de411a6ceaf8_best.jpg',
	 *            1 => 'https://img.shopstyle-cdn.com/pim/0c/a3/0ca3e53759d828c31344e7ce89ba737a_best.jpg',
	 *            2 => 'https://img.shopstyle-cdn.com/pim/4d/19/4d19f71bd0c3bd0de93d70637731649c_best.jpg',
	 *            3 => 'https://img.shopstyle-cdn.com/pim/79/e5/79e5d313e4db5796007ca3ce4c5e1309_best.jpg',
	 *            4 => 'https://img.shopstyle-cdn.com/pim/95/e5/95e5c000cbd249e572b86ecfcd36ba20_best.jpg',
	 *        ],
	 *    'LowerPriceProducts' =>
	 *        [
	 *        ],
	 *    'SameBrandProducts'  => null,
	 *    'SimilarProducts'    =>
	 *        [
	 *        ],
	 *    'ChangedUrlDetails'  => null,
	 *    'InStock'            => true,
	 *    'Onsale'             => false,
	 *    'Categories'         => 'Outerwear',
	 *    'Colors'             =>
	 *        [
	 *            0 => 'Blue',
	 *            1 => 'Beige',
	 *        ],
	 *    'Sizes'              =>
	 *        [
	 *            0 => 'Large',
	 *            1 => 'Medium',
	 *            2 => 'Small',
	 *            3 => 'X-Small',
	 *        ],
	 * ];
	 *
	 * @param $product
	 *
	 * @return array
	 */
	private function map_product_data( $product ) {
		$updated_product                       = [];
		$updated_product['ItemID']             = intval( $product['id'] );
		$updated_product['ItemGuID']           = intval( $product['id'] );
		$updated_product['Brand']              = ! empty( $product['brand']['name'] ) ? sanitize_text_field( $product['brand']['name'] ) : '';
		$updated_product['Title']              = ! empty( $product['name'] ) ? sanitize_text_field( $product['name'] ) : '';
		$updated_product['Price']              = sanitize_text_field( $this->get_product_price( $product ) );
		$updated_product['Source']             = ! empty( $product['clickUrl'] ) ? esc_url( $product['clickUrl'] ) : '';
		$updated_product['Description']        = ! empty( $product['description'] ) ? sanitize_text_field( $product['description'] ) : '';
		$updated_product['Materials']          = [];
		$updated_product['ImageUrls']          = $this->pluck_list_items( $product['image']['sizes']['XLarge'], 'url', 'esc_url' );
		$updated_product['LowerPriceProducts'] = [];
		$updated_product['SameBrandProducts']  = [];
		$updated_product['SimilarProducts']    = [];
		$updated_product['ChangedUrlDetails']  = null;
		$updated_product['InStock']            = ! empty( $product['inStock'] ) ? (bool) $product['inStock'] : false;
		$updated_product['Onsale']             = ! empty( $product['salePriceLabel'] );
		$updated_product['Categories']         = $this->pluck_list_items( $product['categories'], 'name', 'sanitize_text_field', true );
		$updated_product['Colors']             = $this->pluck_list_items( $product['colors'], 'name' );
		$updated_product['Sizes']              = $this->pluck_list_items( $product['sizes'], 'name' );
		$updated_product['productDataSource']  = 'shopstyle';

		return $updated_product;
	}

	/**
	 * Get either the sale price or the regular price, using the label
	 * as it includes '$'.
	 *
	 * @param array $product The product data.
	 *
	 * @return string
	 */
	private function get_product_price( $product ) {
		return ! empty( $product['salePriceLabel'] ) ? $product['salePriceLabel'] : $product['priceLabel'];
	}

	/**
	 * Get new array values based on key.
	 *
	 * @param array  $array     The input array.
	 * @param string $field     The field to look for.
	 * @param string $function  The callback function to sanitize the array values.
	 * @param bool   $implode   Need a list instead of array?
	 * @param string $separator Implode separator.
	 *
	 * @return array|string
	 */
	private function pluck_list_items( $array = [], $field, $function = 'sanitize_text_field', $implode = false, $separator = ', ' ) {
		$results = [];

		if ( is_array( $array ) ) {
			if ( isset( $array[ $field ] ) ) {
				$results[] = $array[ $field ];
			} else {
				foreach ( $array as $key => $value ) {
					if ( is_array( $value ) && $field !== $key ) {
						$new_results = $this->pluck_list_items( $value, $field, $function, $implode, $separator );
						if ( ! is_array( $new_results ) ) {
							$new_results = [ $new_results ];
						}
						$results = array_merge( $results, $new_results );
					}
				}
			}
		}

		$results = array_map( $function, $results );

		if ( ! empty( $results ) ) {
			if ( true === $implode ) {
				return implode( $separator, $results );
			} else {
				return $results;
			}
		}

		return $results;
	}

	/**
	 * Before storing all products that have been added, update a specific
	 * db store of just ShopStyle added products to these aren't overwritten
	 * when the Frenzy API updates products via CRON.
	 *
	 * @param int   $post_id The post id.
	 * @param array $product The product data.
	 *
	 * @return array
	 */
	public function update_shopsimilar_only_product_additions( $post_id, $product ) {
		$existing_shopsimilar_products = get_post_meta( $post_id, 'frenzy-shopstyle-added-products', true );
		if ( empty( $existing_shopsimilar_products ) ) {
			$existing_shopsimilar_products = [];
		}
		array_push( $existing_shopsimilar_products, $product );
		update_post_meta( $post_id, 'frenzy-shopstyle-added-products', $existing_shopsimilar_products );

		return $existing_shopsimilar_products;
	}
}
