<?php

namespace Frenzy\Subscriber;

use DOMDocument;
use Frenzy\Helpers\HelperFunctions;
use Frenzy\Inc\FetchTaxonomyFetchProduct;
use Frenzy\PluginAPI\AbstractSubscriber;
use Frenzy\PluginAPI\HookSubscriberInterface;

/**
 * Builds the shop widget that displays the products for the users.
 */
class ShopWidgetSubscriber extends AbstractSubscriber implements HookSubscriberInterface {

	/**
	 * @var string $pluginPath The path to the plugin on disk.
	 * @access private
	 */
	private $pluginPath;

	/**
	 * @var string $widgetAssets A reference to where the widget's assets are associated with this plugin.
	 * @access private
	 */
	private $widgetAssets;

	/**
	 * Initializes the plugin by setting its path and the reference to the widget's assets.
	 */
	public function __construct( $pluginPath ) {
		$this->pluginPath   = $pluginPath;
		$this->widgetAssets = dirname( dirname( plugin_dir_url( __FILE__ ) ) ) . '/assets/';
	}

	/**
	 * {@inheritdoc}
	 */
	public static function getHooks() {
		return array(
			'the_content'         => array( 'addTemplate', 99 ),
			'post_thumbnail_html' => array( 'getFeaturedImage', 99 ),
		);
	}

	/**
	 * Determines the place (bottom or top) at which the widget should be located.
	 */
	protected function placeButton() {
		echo HelperFunctions::get_shop_button_vertical_location();
	}

	/**
	 * Determines the place (bottom or top) at which the widget should be located.
	 */
	protected function placeButtonHorizontal() {
		echo HelperFunctions::get_shop_button_horizontal_location();
	}

	/**
	 * Outputs a class to modify the functionality of when the 'Shop' button is shown.
	 */
	protected function outputShopHoverOrAlwaysClass() {
		$class       = '';
		$show_always = HelperFunctions::get_frenzy_shop_hover_or_always();
		$class       = 'hover-only';
		if ( 0 !== $show_always ) {
			$class = 'always-show-shop-btn open';
		}
		echo esc_attr( $class );
	}

	/**
	 * If Frenzy is enabled, includes the widgets in the content.
	 *
	 * @param string $content The content that makes up the post.
	 *
	 * @return string
	 */
	public function addTemplate( $content ) {
		if ( defined( 'DOING_AJAX' ) || wp_doing_ajax() || defined( 'REST_REQUEST' ) || is_admin() ) {
			return $content;
		}

		if ( ! $this->fetchEnabledOnPost() ) {
			return $content;
		}

		$newContent = '<span class="frenzy-post-id" style="display:none;" data-value="' . esc_attr( get_the_ID() ) . '"></span>';
		$newContent .= '<span class="frenzy-permalink" style="display:none;" data-value="' . esc_url( get_permalink( get_the_ID() ) ) . '"></span>';
		$newContent .= $this->addImageAttributes( $content );

		// If we didn't add image attributes, return the unchanged content.
		if ( false === strpos( $newContent, 'data-frenzy' ) ) {
			return $content;
		}

		include_once $this->pluginPath . 'views/templates/shop-widget.php';

		return $newContent;
	}

	function fetchEnabledOnPost() {
		$products = FetchTaxonomyFetchProduct::get_products_attached_to_post( get_the_ID() );
		$enabled  = false;
		if ( ! empty( $products ) ) {
			$enabled = true;
		}

		return $enabled;
	}

	/**
	 * @param string $content The content of the post.
	 *
	 * @return string A modified version of the featured image with the data-frenzy attribute added to the post.
	 */
	public function getFeaturedImage( $content ) {
		global $wp_query;
		if ( isset( $wp_query->in_the_loop ) && ! $wp_query->in_the_loop ) {
			return $content;
		}

		return $this->addImageAttributes( $content, false );
	}

	private function addImageAttributes( $content ) {
		$original_content = $content;

		$content = mb_convert_encoding( $content, 'HTML-ENTITIES', 'UTF-8' );
		if ( empty( $content ) ) {
			return $original_content;
		}

		$document        = new DOMDocument();
		$found_something = false;

		libxml_use_internal_errors( true );
		$images = [];

		if ( ! empty( $content ) ) {
			$document->loadHTML( utf8_decode( $content ) );
			$images = $document->getElementsByTagName( 'img' );
		}

		foreach ( $images as $image ) {
			$image->setAttribute( 'data-frenzy', 'true' );
			$image->setAttribute( 'data-post-id', get_the_ID() );
			$class = $image->getAttribute( 'class' );
			if (
				empty( $class ) ||
				(
					false === strpos( $class, 'fetch-product-grid-image' ) &&
					false === strpos( $class, 'fetch-product-carousel-image' )
				)
			) {
				$this->wrapImage( $document, $image );
			}
			$found_something = true;
		}

		// Check if Jetpack Gallery is in use.
		if ( false === $found_something ) {
			$document->getElementsByTagName( 'p' );
			$paragraphs = $document->getElementsByTagName( 'div' );
			foreach ( $paragraphs as $element ) {
				$classy = $element->getAttribute( 'class' );
				if ( strpos( $classy, 'jetpack-slideshow' ) !== false ) {
					$element->setAttribute( 'data-frenzy', 'true' );
					$element->setAttribute( 'data-post-id', get_the_ID() );
				}
			}
		}

		return $document->saveHTML();
	}

	/**
	 * This function is used to wrap individual images in a paragraph element
	 * if their next sibling is an image because each image needs to be wrapped
	 * in its own paragraph element to properly render the carousel.
	 *
	 * @param $document The DOM Document which is to be rendered.
	 * @param $image    The image to wrap with the new paragraph element.
	 */
	private function wrapImage( $document, $image ) {
		if ( null === $image || empty( $image->parentNode ) ) {
			return;
		}
		$wrapper = $document->createElement( 'p' );
		$wrapper->setAttribute( 'class', 'frenzy-image' );

		$image->parentNode->replaceChild( $wrapper, $image );
		if ( null !== $image ) {
			$wrapper->appendChild( $image );
		}
	}
}
